@@ -18,6 +18,11 @@
 from __future__ import annotations
 import inspect
 import typing
+import functools
+from multidict import CIMultiDict
+
+from lightbulb import context
+from lightbulb import errors
 
 
 _CommandT = typing.TypeVar("_CommandT", bound="Command")
@@ -67,10 +72,53 @@ def _bind_prototype(instance: typing.Any, command_template: _CommandT):
     return typing.cast(_CommandT, prototype)
 
 
+class SignatureInspector:
+    """
+    Contains information about the arguments that a command takes when
+    it is invoked.
+
+    Args:
+        command (:obj:`~.commands.Command`): The command to inspect the arguments of.
+    """
+
+    def __init__(self, command: Command) -> None:
+        self.command = command
+        self.has_self = isinstance(command, _BoundCommandMarker)
+        self.args = {}
+        for index, (name, arg) in enumerate(
+            inspect.signature(command._callback).parameters.items()
+        ):
+            self.args[name] = self.parse_arg(index, arg)
+        self.max_args = sum(not arg["ignore"] for arg in self.args.values())
+        self.min_args = sum(
+            not arg["ignore"] and arg["required"] for arg in self.args.values()
+        )
+        self.has_max_args = not any(
+            a.kind == inspect.Parameter.VAR_POSITIONAL
+            for a in inspect.signature(command._callback).parameters.values()
+        )
+
+    def parse_arg(self, index, arg):
+        details = {}
+        if index == 0:
+            details["ignore"] = True
+        elif index == 1 and self.has_self:
+            details["ignore"] = True
+        else:
+            details["ignore"] = False
+
+        details["argtype"] = arg.kind
+        details["annotation"] = arg.annotation
+        details["required"] = (arg.default is arg.empty) or (
+            arg.kind == 3
+        )  # var positional
+        return details
+
+
 class Command:
     """
     A command that can be invoked by a user. When invoked, the callback
-    will be called with a set argument ctx, an instance of the :obj:`.context.Context`
+    will be called with a set argument ctx, an instance of the :obj:`~.context.Context`
     class, and any other arguments supplied by the user.
 
     Args:
@@ -94,28 +142,6 @@ class Command:
         self.method_name: typing.Optional[str] = None
         self.parent = parent
 
-        signature = inspect.signature(callback)
-        self._has_max_args = not any(
-            a.kind == inspect.Parameter.VAR_POSITIONAL
-            for a in signature.parameters.values()
-        )
-
-        self._max_args: int = 0
-        self._min_args: int = -1
-
-        has_self = False
-
-        for i, (name, param) in enumerate(signature.parameters.items()):
-            if name == "self" and i == 0:
-                has_self = True
-                continue
-            if param.default == inspect.Parameter.empty:
-                self._min_args += 1
-
-            # Skip the context, also skip counting self if it was present...
-            if has_self and i > 1 or i > 0:
-                self._max_args += 1
-
     def __get__(
         self: _CommandT, instance: typing.Any, owner: typing.Type[typing.Any]
     ) -> _CommandT:
@@ -124,6 +150,16 @@ class Command:
     def __set_name__(self, owner, name):
         self.method_name = name
 
+    @functools.cached_property
+    def arg_details(self):
+        """
+        An inspection of the arguments that a command takes.
+
+        Returns:
+            :obj:`~.commands.SignatureInspector`: Details about the command's arguments.
+        """
+        return SignatureInspector(self)
+
     @property
     def name(self) -> str:
         """
@@ -137,9 +173,10 @@ class Command:
     @property
     def is_subcommand(self) -> bool:
         """
+        Whether or not the object is a subcommand with a parent group
+
         Returns:
-            :obj:`True` if this object is a subcommand with a parent group,
-            or :obj:`False` otherwise.
+            Union[ :obj:`True`, :obj:`False` ]: If this object is a subcommand with a parent group.
         """
         return self.parent is not None
 
@@ -158,13 +195,13 @@ class Command:
 
     def add_check(self, check_func) -> None:
         """
-        Add a check to an instance of :obj:`.commands.Command` or a subclass. The check passed must
-        be an awaitable function taking a single argument which will be an instance of :obj:`.context.Context`.
+        Add a check to an instance of :obj:`~.commands.Command` or a subclass. The check passed must
+        be an awaitable function taking a single argument which will be an instance of :obj:`~.context.Context`.
         It must also either return a boolean denoting whether or not the check passed,
-        or raise an instance of :obj:`.errors.CheckFailure` or a subclass.
+        or raise an instance of :obj:`~.errors.CheckFailure` or a subclass.
 
         Args:
-            check_func (Callable[ [ :obj:`.context.Context` ], Coroutine[ ``None``, ``None``, :obj:`bool` ] ]): Check to add to the command
+            check_func (Callable[ [ :obj:`~.context.Context` ], Coroutine[ ``None``, ``None``, :obj:`bool` ] ]): Check to add to the command
 
         Returns:
             ``None``
@@ -180,6 +217,28 @@ class Command:
         """
         self._checks.append(check_func)
 
+    async def is_runnable(self, context: context.Context) -> bool:
+        """
+        Run all the checks for the command to determine whether or not it is
+        runnable in the given context.
+
+        Args:
+            context (:obj:`~.context.Context`): The context to evaluate the checks with.
+
+        Returns:
+            :obj:`bool`: If the command is runnable in the context given.
+
+        Raises:
+            :obj:`~.errors.CheckFailure`: If the command is not runnable in the context given.
+        """
+        for check in self._checks:
+            result = await check(context)
+            if result is False:
+                raise errors.CheckFailure(
+                    f"Check {check.__name__} failed for command {self.name}"
+                )
+        return True
+
 
 class Group(Command):
     """
@@ -187,15 +246,17 @@ class Group(Command):
     for subcommands which can be registered to the group and invoked as separate commands.
 
     Args:
-        *args: The args passed to :obj:`.commands.Command` in its constructor
+        *args: The args passed to :obj:`~.commands.Command` in its constructor
 
     Keyword Args:
-        **kwargs: The kwargs passed to :obj:`.commands.Command` in its constructor
+        insensitive_commands (:obj:`bool`): Whether or not this group's subcommands should be case insensitive. Defaults to ``False``.
+        **kwargs: The kwargs passed to :obj:`~.commands.Command` in its constructor
     """
 
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args, insensitive_commands: bool = False, **kwargs) -> None:
         super().__init__(*args, **kwargs)
-        self.subcommands = {}
+        self.insensitive_commands = insensitive_commands
+        self.subcommands = {} if not self.insensitive_commands else CIMultiDict()
 
     def _resolve_subcommand(
         self, args
@@ -203,10 +264,10 @@ class Group(Command):
         this = self
 
         args.pop(0)
-
         while isinstance(this, Group) and args:
+            invoked_with = args[0].casefold() if this.insensitive_commands else args[0]
             try:
-                this = this.subcommands[args[0]]
+                this = this.subcommands[invoked_with]
             except KeyError:
                 break
             else:
@@ -222,7 +283,7 @@ class Group(Command):
             name (:obj:`str`): The name of the command to get the object for.
 
         Returns:
-            Optional[ :obj:`.commands.Command` ]: command object registered to that name.
+            Optional[ :obj:`~.commands.Command` ]: command object registered to that name.
         """
         return self.subcommands.get(name)
 
@@ -270,13 +331,22 @@ class Group(Command):
         return decorate
 
     def group(self, **kwargs):
+        """
+        A decorator that registers a callable as a subgroup for the command group.
+
+        Keyword Args:
+            allow_extra_arguments (:obj:`bool`): Whether or not the handler should raise an error if the command is run
+                with more arguments than it requires. Defaults to True.
+            name (:obj:`str`): Optional name of the command. Defaults to the name of the function if not specified.
+            aliases (Optional[ Iterable[ :obj:`str` ] ]): An iterable of aliases which can also invoke the command.
+        """
         kwargs["cls"] = type(self)
         return self.command(**kwargs)
 
 
 def command(**kwargs):
     """
-    A decorator to convert a coroutine into a :obj:`.commands.Command` object.
+    A decorator to convert a coroutine into a :obj:`~.commands.Command` object.
 
     Keyword Args:
         name (Optional[ :obj:`str` ]): Name to register the command to. Defaults to the name of the coroutine.
@@ -299,7 +369,7 @@ def command(**kwargs):
 
 def group(**kwargs):
     """
-    A decorator to convert a coroutine into a :obj:`.commands.Group` object.
+    A decorator to convert a coroutine into a :obj:`~.commands.Group` object.
 
     Keyword Args:
         name (Optional[ :obj:`str` ]): Name to register the command to. Defaults to the name of the coroutine.
@@ -315,6 +385,7 @@ def group(**kwargs):
             name,
             kwargs.get("allow_extra_arguments", True),
             kwargs.get("aliases", []),
+            insensitive_commands=kwargs.get("insensitive_commands", False),
         )
 
     return decorate
