@@ -32,11 +32,11 @@ class Context:
     The context a command was invoked under.
 
     Args:
-        bot (:obj:`.command_handler.BotWithHandler`): The bot instance that received the command.
+        bot (:obj:`~.command_handler.BotWithHandler`): The bot instance that received the command.
         message (:obj:`hikari.models.messages.Message`): The message the context was created from.
         prefix (:obj:`str`): The prefix used in the context.
         invoked_with (:obj:`str`): The name or alias used to invoke a command.
-        command (:obj:`.commands.Command`): The command that was invoked.
+        command (:obj:`~.commands.Command`): The command that was invoked.
 
     Note:
         For information on types for the various properties see :obj:`hikari.models.messages.Message`.
@@ -82,7 +82,7 @@ class Context:
 
     async def reply(self, *args, **kwargs) -> messages.Message:
         """
-        Alias for ctx.message.reply(...).
+        Alias for ``ctx.message.reply(...)``.
         Replies to the message in the current context.
 
         Args:
