@@ -39,7 +39,7 @@ class Plugin:
         .. code-block:: python
 
             import lightbulb
-            from lightbulb import plugins
+            from lightbulb import plugins, commands
 
             bot = lightbulb.Bot(token="token_here", prefix="!")
 
@@ -65,3 +65,5 @@ class Plugin:
                 if not member.is_subcommand:
                     # using self here to now get the bound command.
                     self.commands[member.name] = getattr(self, name)
+                    for alias in member._aliases:
+                        self.commands[alias] = self.commands[member._name]
